/*
 * kernel.c
 *
 * This file contains the boot stage of the kernel.
 * This file belongs to PGOS.
 *
 */

#include <intr.h>
#include <panic.h>
#include <term/kprint.h>
#include <term/vga.h>

#ifdef __linux__
#error "Run `make toolchain'! You're using your host compiler."
#endif

#ifndef __i386__
#error "Your toolchain seems to be 64-bit, run `make toolchain' for an i686 compiler."
#endif

void kernel_main () {
  uint32_t eax;
  uint32_t ebx;
  
  /* Have to store EAX & EBX before we accidentally alter them */
  __asm__ __volatile__ ("movl %%eax, %0" : "=m" (eax) : : );
  __asm__ __volatile__ ("movl %%ebx, %0" : "=m" (ebx) : : );

  if (eax != 0x2BADB002) {
    panic("We weren't booted by a Multiboot-compliant loader!");
  }
  
  terminal_init();
  terminal_clear();
  
  terminal_write("Setting up interrupts...\n");
  load_default_idt();

  __asm__ __volatile__ ( "int $0" );

  terminal_write("Tested int 0.\n");
}
