/*
 * acpi.c
 *
 * This file contains a kernel panic routine.
 * This file belongs to PGOS.
 *
 */

#include <panic.h>
#include <term/kprint.h>
#include <term/vga.h>

/* PANIC!!! Do NOTHING fancy, we're in danger. Dump the registers and hang. */
void panic (const char *message) {
  uint32_t eax;
  uint32_t ebx;
  uint32_t ecx;
  uint32_t edx;
  uint32_t esp;
  uint32_t ebp;
  uint32_t esi;
  uint32_t edi;

  uint32_t eip;
  uint32_t eflags;

  __asm__ __volatile__ ("movl %%eax, %0" : "=m" (eax));
  __asm__ __volatile__ ("movl %%ebx, %0" : "=m" (ebx));
  __asm__ __volatile__ ("movl %%ecx, %0" : "=m" (ecx));
  __asm__ __volatile__ ("movl %%edx, %0" : "=m" (edx));
  __asm__ __volatile__ ("movl %%esp, %0" : "=m" (esp));
  __asm__ __volatile__ ("movl %%ebp, %0" : "=m" (ebp));
  __asm__ __volatile__ ("movl %%esi, %0" : "=m" (esi));
  __asm__ __volatile__ ("movl %%edi, %0" : "=m" (edi));

  __asm__ ("movl (%%esp), %%eax\n\t"
	   "movl %%eax, %0" : "=m" (eip));

  __asm__ ("pushfl\n\t"
	   "pop %%eax\n\t"
	   "movl %%eax, %0" : "=m" (eflags));
  
  terminal_write("\n\nGuru Meditation Error: ");
  terminal_write(message);

  terminal_write("\n\nRegister dump: \nEAX 0x");
  kprint_hex(eax,8);
  terminal_write(" EBX 0x");
  kprint_hex(ebx,8);
  terminal_write(" ECX 0x");
  kprint_hex(ecx,8);
  terminal_write(" EDX 0x");
  kprint_hex(edx,8);
  terminal_write("\nESP 0x");
  kprint_hex(esp,8);
  terminal_write(" EBP 0x");
  kprint_hex(ebp,8);
  terminal_write(" ESI 0x");
  kprint_hex(esp,8);
  terminal_write(" EDI 0x");
  kprint_hex(edi,8);
  terminal_write("\nEIP 0x");
  kprint_hex(eip,8);
  terminal_write(" EFLAGS 0x");
  kprint_hex(eflags,8);
  
  __asm__ (".L%=:\n\t"
	   "cli\n\t"
	   "hlt\n\t"
	   "jmp .L%=" : : :);
}
