/*
 * kprint.c
 *
 * This file contains routines for printing hex, etc.
 * This file belongs to PGOS.
 *
 */

#include <term/kprint.h>
#include <term/vga.h>

const char size_prefixes[6] = {'K', 'M', 'G', 'T', 'P', 'E'};

char nibble_th (uint8_t nibble) {
  return (nibble < 10) ? '0' + nibble : 'a' + nibble - 0x0A;
}

void kprint_hex (uint32_t val, size_t len) {
  char buf[8] = {0, 0, 0, 0, 0, 0, 0, 0};
  size_t cind;
  
  for (cind = 0; val > 0; cind++) {
    buf[len-1-cind] = nibble_th(val & 0x0F);
    val >>= 4;
  }
  for (; cind < len; cind++) {
    buf[len-1-cind] = '0';
  }
  
  terminal_write_fixed((char*) buf, 8);
}

void kprint_size (uint32_t val_lo, uint32_t val_hi) {
  char *out;
  size_t cind;
  
  size_t prefix = 0;
  char buf[4] = {0, 0, 0, 0};
  size_t diff = 4;
  
  if (val_hi > 0) {
    prefix = 3;
    val_lo >>= 30;
    val_lo += 4*val_hi;
  } else if (val_lo == 0) {
    terminal_putchar('0');
    return;
  }
  
  while ((val_lo > 4096) && (prefix < 6)) {
    val_lo >>= 10;
    prefix++;
  }

  for (cind = 0; val_lo > 0; cind++) {
    buf[3-cind] = '0' + (val_lo % 10);
    val_lo /= 10;
  }

  out = (char*) buf;
  while (out[0] == 0) {
    out++;
    diff--;
  }
  
  terminal_write_fixed(out, diff);
  terminal_putchar(' ');
  if (prefix > 0) {
    terminal_putchar(size_prefixes[prefix-1]);
    terminal_putchar('i');
  }
}
